local GuildUninvite = GuildUninvite
local GetGuildRosterInfo = GetGuildRosterInfo
local GetGuildRosterLastOnline = GetGuildRosterLastOnline
local gsub = string.gsub
local find = string.find
local match = string.match

local Roster = {}
local GuildRanks = {}
local NumGuildRanks = 0
local Requested = false
local GuildMemberCount = 0
local TimeMap = {"Years", "Months", "Days", "Hours"}
local ActiveCheckOption = "Months"
local ActiveNumberOption = 2
local ActiveRankIndex = 3
local RemoveCount = 0

local TimerParent1 = CreateFrame("Frame"):CreateAnimationGroup()
local Timer1 = TimerParent1:CreateAnimation("Alpha")
Timer1:SetDuration(1.5)

local TimerParent2 = CreateFrame("Frame"):CreateAnimationGroup()
local Timer2 = TimerParent2:CreateAnimation("Alpha")
Timer2:SetDuration(10)

local StripRealm = function(name)
	if find(name, "-") then
		return match(name, "(%a+)-")
	else
		return name
	end
end

local Print = function(...)
	print("|cff119affGuild Management|r:", ...)
end

local DontKick = function(note)
	if (note == "DK") then
		return true
	else
		return false
	end
end

local BuildGuildRoster = function()
	local Name, Rank, RankIndex, Level, Years, Months, Days, Hours, RealName, OfficerNote, _
	GuildMemberCount = GetNumGuildMembers()
	
	for i = 1, GuildMemberCount do
		Name, Rank, RankIndex, Level, _, _, _, OfficerNote = GetGuildRosterInfo(i)
		Years, Months, Days, Hours = GetGuildRosterLastOnline(i)
		Years = Years and Years or 0
		Months = Months and Months or 0
		Days = Days and Days or 0
		Hours = Hours and Hours or 0
		
		if (not GuildRanks[RankIndex]) then -- Build the GuildRanks table
			GuildRanks[RankIndex] = Rank
			NumGuildRanks = NumGuildRanks + 1
		end
		
		if Name then
			RealName = StripRealm(Name)
			
			Roster[i] = {Name = RealName, Rank = Rank, RankIndex = RankIndex, Level = Level, Years = Years, Months = Months, Days = Days, Hours = Hours, GuildIndex = i, Note = OfficerNote}
		end
	end
end

local UpdateGuildMembers = function()
	Requested = true
	GuildRoster() -- Tell the server to send updated guild information
	TimerParent1:Play()
end

-- Sometimes we don't get the information we requested through GUILD_ROSTER_UPDATE, schedule another call
local CheckReceivedInfo = function()
	if (GuildMemberCount == 0) then
		Print("Updated information was not received from the server. Trying again in 10 seconds.")
		TimerParent2:Play()
	end
end

local ScheduledUpdate = function()
	UpdateGuildMembers()
end

Timer1:SetScript("OnFinished", CheckReceivedInfo)
Timer2:SetScript("OnFinished", ScheduledUpdate)

-- GUI
local Font = "Interface\\Addons\\GuildManagement\\Engschrift.ttf"
local FontColor = {220/255, 220/255, 220/255}

local Backdrop = {
	bgFile = "Interface\\AddOns\\GuildManagement\\Blank.tga",
	edgeFile = "Interface\\AddOns\\GuildManagement\\Blank.tga",
	tile = false, tileSize = 0, edgeSize = 1,
	insets = {left = 1, right = 1, top = 1, bottom = 1},
}

local BackdropBorder = {	
	edgeFile = "Interface\\AddOns\\GuildManagement\\Blank.tga",
	edgeSize = 1,
	insets = {left = 0, right = 0, top = 0, bottom = 0},
}

local SetTemplate = function(self)
	self:SetBackdrop(Backdrop)
	self:SetBackdropBorderColor(67/255, 77/255, 91/255)
	self:SetBackdropColor(29/255, 38/255, 51/255)
end

local SetTemplateDark = function(self)
	self:SetBackdrop(Backdrop)
	self:SetBackdropBorderColor(67/255, 77/255, 91/255)
	self:SetBackdropColor(23/255, 30/255, 41/255)
end

local UpdateGuildCount = function()
	GMUI_GUI.TopLabel:SetText(select(1, GetGuildInfo("player")) .. " - " .. GuildMemberCount .. " members")
end

local UpdateCheckOptionText = function()
	GMUI_GUI.NumberOfXLabel:SetText("Set " .. ActiveCheckOption:lower() .. ":")
end

local UpdateBottomLabel = function()
	GMUI_GUI.BottomLabel:SetText(RemoveCount .. " players will be removed")
end

local GetNumEligableKicks = function()
	RemoveCount = 0
	
	for Index, Info in pairs(Roster) do
		if (Info[ActiveCheckOption] >= ActiveNumberOption) and (Info.RankIndex > ActiveRankIndex) and (Info.Note and not DontKick(Info.Note)) then
			RemoveCount = RemoveCount + 1
		end
	end
	
	UpdateBottomLabel()
end

local EB_OnClick = function(self)
	self:SetAutoFocus(true)
end

local EB_OnEscapePressed = function(self)
	self:SetAutoFocus(false)
	self:ClearFocus()
end

local EB_OnEnterPressed = function(self)
	self:SetAutoFocus(false)
	self:ClearFocus()
	
	ActiveNumberOption = tonumber(self:GetText()) or 0
	GetNumEligableKicks()
end

local EB_OnEditFocusLost = function(self)
	self:SetAutoFocus(false)
end

local Button_OnEnter = function(self)
	self:SetBackdropBorderColor(100/255, 100/255, 150/255)
end

local Button_OnLeave = function(self)
	self:SetBackdropBorderColor(67/255, 77/255, 91/255)
end

local RemoveEligablePlayers = function()
	if (not CanGuildRemove()) then
		Print("You don't have privelages to remove players from the guild.")
		return
	end
	
	for Index, Info in pairs(Roster) do
		if (Info[ActiveCheckOption] >= ActiveNumberOption) and (Info.RankIndex > ActiveRankIndex) and (Info.Note and not DontKick(Info.Note)) then
			GuildUninvite(Info.Name)
		end
	end
	
	Roster = table.wipe(Roster) -- Wipe the roster, rebuild it
	GMUI_GUI.TopLabel:SetText(select(1, GetGuildInfo("player")) .. " - " .. (GuildMemberCount - RemoveCount) .. " members")
	GMUI_GUI.BottomLabel:SetText("0 players will be removed")
	ToggleGMUI_GUI()
end

local Button_OnClick = function(self)
	GMUI_Popup:Show()
end

local LoadGMUI_GUI = function()
	local GUI = CreateFrame("Frame", "GMUI_GUI", UIParent)
	GUI:SetSize(200, 117)
	GUI:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	GUI:SetMovable(true)
	SetTemplate(GUI)
	
	local Top = CreateFrame("Frame", nil, GUI)
	Top:SetSize(200, 21)
	Top:SetPoint("TOP", GUI, "TOP", 0, 0)
	SetTemplateDark(Top)
	GUI.Top = Top
	
	Top:EnableMouse(true)
	Top:RegisterForDrag("LeftButton")
	Top:SetScript("OnDragStart", function() GUI:StartMoving() end)
	Top:SetScript("OnDragStop", function() GUI:StopMovingOrSizing() end)
	
	local Close = CreateFrame("Button", nil, Top)
	Close:SetSize(21, 21)
	Close:SetPoint("TOPRIGHT", GUI, "TOPRIGHT", 0, 0)
	Close:SetFrameStrata("MEDIUM")
	SetTemplateDark(Close)
	Close:SetScript("OnMouseUp", function(self)
		GMUI_GUI:Hide()
	end)
	Close:SetScript("OnEnter", function(self) self.X:SetTextColor(1, 1, 1) end)
	Close:SetScript("OnLeave", function(self) self.X:SetTextColor(unpack(FontColor)) end)
	
	Close.X = Close:CreateFontString(nil, "OVERLAY")
	Close.X:SetPoint("CENTER", Close, "CENTER", 1, -1)
	Close.X:SetFont(Font, 14)
	Close.X:SetTextColor(unpack(FontColor))
	Close.X:SetText("X")
	Close.X:SetShadowOffset(1.25, -1.25)
	Close.X:SetShadowColor(0, 0, 0)
	
	local BottomBase = CreateFrame("Frame", nil, GUI)
	BottomBase:SetSize(200, 21)
	BottomBase:SetPoint("TOP", GUI, "BOTTOM", 0, 1)
	SetTemplateDark(BottomBase)
	
	local Bottom = CreateFrame("Frame", nil, GUI)
	Bottom:SetSize(200, 21)
	Bottom:SetPoint("TOP", GUI, "BOTTOM", 0, 1)
	--SetTemplateDark(Bottom)
	
	GUI.TopLabel = Top:CreateFontString(nil, "OVERLAY")
	GUI.TopLabel:SetPoint("TOPLEFT", Top, "TOPLEFT", 4, -4)
	GUI.TopLabel:SetFont(Font, 16)
	GUI.TopLabel:SetTextColor(unpack(FontColor))
	GUI.TopLabel:SetText(select(1, GetGuildInfo("player")))
	GUI.TopLabel:SetShadowOffset(1.25, -1.25)
	GUI.TopLabel:SetShadowColor(0, 0, 0)
	
	GUI.BottomLabel = Bottom:CreateFontString(nil, "OVERLAY")
	GUI.BottomLabel:SetPoint("TOPLEFT", Bottom, "TOPLEFT", 4, -4)
	GUI.BottomLabel:SetFont(Font, 14)
	GUI.BottomLabel:SetTextColor(unpack(FontColor))
	GUI.BottomLabel:SetText("")
	GUI.BottomLabel:SetShadowOffset(1.25, -1.25)
	GUI.BottomLabel:SetShadowColor(0, 0, 0)
	
	GUI.RemoveLabel = Top:CreateFontString(nil, "OVERLAY")
	GUI.RemoveLabel:SetPoint("TOPLEFT", Top, "BOTTOMLEFT", 4, -5)
	GUI.RemoveLabel:SetFont(Font, 16)
	GUI.RemoveLabel:SetTextColor(unpack(FontColor))
	GUI.RemoveLabel:SetText("Time offline:")
	GUI.RemoveLabel:SetShadowOffset(1.25, -1.25)
	GUI.RemoveLabel:SetShadowColor(0, 0, 0)
	
	local DropDown = CreateFrame("Frame", nil, GUI)
	DropDown:SetSize(120, 20)
	DropDown:SetPoint("TOPRIGHT", Top, "BOTTOMRIGHT", -4, -3)
	SetTemplateDark(DropDown)
	
	DropDown.Label = DropDown:CreateFontString(nil, "OVERLAY")
	DropDown.Label:SetPoint("LEFT", DropDown, "LEFT", 4, -1.5)
	DropDown.Label:SetFont(Font, 16)
	DropDown.Label:SetTextColor(unpack(FontColor))
	DropDown.Label:SetShadowOffset(1.25, -1.25)
	DropDown.Label:SetShadowColor(0, 0, 0)
	
	DropDown.Button = CreateFrame("Frame", nil, DropDown)
	DropDown.Button:SetSize(20, 20)
	DropDown.Button:SetPoint("RIGHT", DropDown, "RIGHT", 0, 0)
	DropDown.Button.Pos = -1
	SetTemplate(DropDown.Button)
	
	DropDown.Button.Label = DropDown.Button:CreateFontString(nil, "OVERLAY")
	DropDown.Button.Label:SetPoint("CENTER", DropDown.Button, "CENTER", 1, -1.5)
	DropDown.Button.Label:SetFont(Font, 14)
	DropDown.Button.Label:SetTextColor(unpack(FontColor))
	DropDown.Button.Label:SetShadowOffset(1.25, -1.25)
	DropDown.Button.Label:SetShadowColor(0, 0, 0)
	DropDown.Button.Label:SetText("V")
	
	DropDown.Options = CreateFrame("Frame", nil, DropDown)
	DropDown.Options:SetSize(120, (4 * 20))
	DropDown.Options:SetPoint("TOP", DropDown, "BOTTOM", 0, -3)
	DropDown.Options:SetFrameStrata("HIGH")
	DropDown.Options:Hide()
	SetTemplate(DropDown.Options)
	
	DropDown.Button:SetScript("OnMouseUp", function(self)
		if (self.Pos == 1) then
			DropDown.Options:Hide()
			DropDown.Button.Label:SetText("V")
			
			self.Pos = -1
		else
			DropDown.Options:Show()
			DropDown.Button.Label:SetText("^")
			
			self.Pos = 1
		end
	end)
	
	for i = 1, 4 do
		DropDown.Options[i] = CreateFrame("Frame", nil, DropDown.Options)
		DropDown.Options[i]:SetSize(120, 20)
		DropDown.Options[i]:SetPoint("TOP", DropDown.Options, "TOP", 0, -(20 * (i - 1)))
		DropDown.Options[i].Index = i
		
		DropDown.Options[i]:SetScript("OnEnter", function(self) self.bg:SetTexture(67/255, 77/255, 91/255, 1) end)
		DropDown.Options[i]:SetScript("OnLeave", function(self) self.bg:SetTexture(0, 0, 0, 0) end)
		
		DropDown.Options[i]:SetScript("OnMouseUp", function(self)
			ActiveCheckOption = TimeMap[self.Index]
			DropDown.Label:SetText(self.Label:GetText())
			DropDown.Options:Hide()
			DropDown.Button.Label:SetText("V")
			DropDown.Button.Pos = -1
			UpdateCheckOptionText()
			GetNumEligableKicks()
		end)
		
		DropDown.Options[i].bg = DropDown.Options[i]:CreateTexture(nil, "OVERLAY")
		DropDown.Options[i].bg:SetSize(118, 20)
		DropDown.Options[i].bg:SetPoint("CENTER", DropDown.Options[i], "CENTER", 0, 0)
		
		DropDown.Options[i].Label = DropDown.Options[i]:CreateFontString(nil, "OVERLAY")
		DropDown.Options[i].Label:SetPoint("LEFT", DropDown.Options[i], "LEFT", 4, -1.5)
		DropDown.Options[i].Label:SetFont(Font, 16)
		DropDown.Options[i].Label:SetTextColor(unpack(FontColor))
		DropDown.Options[i].Label:SetShadowOffset(1.25, -1.25)
		DropDown.Options[i].Label:SetShadowColor(0, 0, 0)
		DropDown.Options[i].Label:SetText(TimeMap[i])
	end
	
	if (not DropDown.Label:GetText()) then
		DropDown.Label:SetText(ActiveCheckOption)
	end
	
	GUI.NumberOfXLabel = Top:CreateFontString(nil, "OVERLAY")
	GUI.NumberOfXLabel:SetPoint("TOPLEFT", Top, "BOTTOMLEFT", 5, -28)
	GUI.NumberOfXLabel:SetFont(Font, 16)
	GUI.NumberOfXLabel:SetTextColor(unpack(FontColor))
	GUI.NumberOfXLabel:SetShadowOffset(1.25, -1.25)
	GUI.NumberOfXLabel:SetShadowColor(0, 0, 0)
	GUI.NumberOfXLabel:SetText("Set months:")
	
	local EditBoxBG = CreateFrame("Frame", nil, GUI)
	EditBoxBG:SetSize(60, 20)
	EditBoxBG:SetPoint("TOPRIGHT", Top, "BOTTOMRIGHT", -4, -26)
	SetTemplateDark(EditBoxBG)
	
	local EditBox = CreateFrame("EditBox", nil, EditBoxBG)
	EditBox:SetSize(60, 20)
	EditBox:SetPoint("LEFT", EditBoxBG, "LEFT", 4, -2)
	EditBox:SetFont(Font, 16)
	EditBox:SetAutoFocus(false)
	EditBox:SetScript("OnMouseUp", EB_OnClick)
	EditBox:SetScript("OnEscapePressed", EB_OnEscapePressed)
	EditBox:SetScript("OnEnterPressed", EB_OnEnterPressed)
	EditBox:SetScript("OnEditFocusLost", EB_OnEditFocusLost)
	EditBox:SetNumeric(true)
	EditBox:SetMaxLetters(4)
	EditBox:SetText("2")
	
	GUI.RankLabel = Top:CreateFontString(nil, "OVERLAY")
	GUI.RankLabel:SetPoint("TOPLEFT", Top, "BOTTOMLEFT", 4, -50)
	GUI.RankLabel:SetFont(Font, 16)
	GUI.RankLabel:SetTextColor(unpack(FontColor))
	GUI.RankLabel:SetShadowOffset(1.25, -1.25)
	GUI.RankLabel:SetShadowColor(0, 0, 0)
	GUI.RankLabel:SetText("Below rank:")
	
	local RemoveButton = CreateFrame("Button", nil, GUI)
	RemoveButton:SetSize(192, 20)
	RemoveButton:SetPoint("BOTTOMLEFT", GUI, "BOTTOMLEFT", 4, 4)
	SetTemplateDark(RemoveButton)
	RemoveButton:SetScript("OnEnter", Button_OnEnter)
	RemoveButton:SetScript("OnLeave", Button_OnLeave)
	RemoveButton:SetScript("OnMouseUp", Button_OnClick)
	
	RemoveButton.Label = RemoveButton:CreateFontString(nil, "OVERLAY")
	RemoveButton.Label:SetPoint("CENTER", RemoveButton, "CENTER", 0, -1.5)
	RemoveButton.Label:SetFont(Font, 16)
	RemoveButton.Label:SetTextColor(unpack(FontColor))
	RemoveButton.Label:SetShadowOffset(1.25, -1.25)
	RemoveButton.Label:SetShadowColor(0, 0, 0)
	RemoveButton.Label:SetText("Remove Players")
	
	-- Confirmation popup
	local Popup = CreateFrame("Frame", "GMUI_Popup", UIParent)
	Popup:SetSize(200, 70)
	Popup:SetPoint("CENTER", UIParent, "CENTER", 0, 150)
	SetTemplate(Popup)
	Popup:Hide()
	
	Popup.Label = Popup:CreateFontString(nil, "OVERLAY")
	Popup.Label:SetPoint("TOP", Popup, "TOP", 0, -5)
	Popup.Label:SetWidth(192)
	Popup.Label:SetFont(Font, 16)
	Popup.Label:SetTextColor(unpack(FontColor))
	Popup.Label:SetShadowOffset(1.25, -1.25)
	Popup.Label:SetShadowColor(0, 0, 0)
	Popup.Label:SetText("Are you sure you want to remove these players?")
	
	Popup.Accept = CreateFrame("Button", nil, Popup)
	Popup.Accept:SetSize(94, 20)
	Popup.Accept:SetPoint("BOTTOMLEFT", Popup, "BOTTOMLEFT", 4, 4)
	SetTemplateDark(Popup.Accept)
	Popup.Accept:SetScript("OnEnter", Button_OnEnter)
	Popup.Accept:SetScript("OnLeave", Button_OnLeave)
	Popup.Accept:SetScript("OnMouseUp", function(self)
		RemoveEligablePlayers() -- Bye bye
		GMUI_Popup:Hide()
	end)
	
	Popup.Accept.Label = Popup.Accept:CreateFontString(nil, "OVERLAY")
	Popup.Accept.Label:SetPoint("CENTER", Popup.Accept, "CENTER", 0, 0)
	Popup.Accept.Label:SetFont(Font, 16)
	Popup.Accept.Label:SetTextColor(unpack(FontColor))
	Popup.Accept.Label:SetShadowOffset(1.25, -1.25)
	Popup.Accept.Label:SetShadowColor(0, 0, 0)
	Popup.Accept.Label:SetText("Accept")
	
	Popup.Cancel = CreateFrame("Button", nil, Popup)
	Popup.Cancel:SetSize(94, 20)
	Popup.Cancel:SetPoint("BOTTOMRIGHT", Popup, "BOTTOMRIGHT", -4, 4)
	SetTemplateDark(Popup.Cancel)
	Popup.Cancel:SetScript("OnEnter", Button_OnEnter)
	Popup.Cancel:SetScript("OnLeave", Button_OnLeave)
	Popup.Cancel:SetScript("OnMouseUp", function(self)
		GMUI_Popup:Hide()
	end)
	
	Popup.Cancel.Label = Popup.Cancel:CreateFontString(nil, "OVERLAY")
	Popup.Cancel.Label:SetPoint("CENTER", Popup.Cancel, "CENTER", 0, 0)
	Popup.Cancel.Label:SetFont(Font, 16)
	Popup.Cancel.Label:SetTextColor(unpack(FontColor))
	Popup.Cancel.Label:SetShadowOffset(1.25, -1.25)
	Popup.Cancel.Label:SetShadowColor(0, 0, 0)
	Popup.Cancel.Label:SetText("Cancel")
	
	local Border = CreateFrame("Frame", nil, GUI)
	Border:SetPoint("TOPLEFT", Top, -1, 1)
	Border:SetPoint("BOTTOMRIGHT", Bottom, 1, -1)
	Border:SetFrameStrata("BACKGROUND")
	Border:SetFrameLevel(1)
	Border:SetBackdrop(BackdropBorder)
	Border:SetBackdropBorderColor(0, 0, 0)
	
	local PopupBorder = CreateFrame("Frame", nil, Popup)
	PopupBorder:SetPoint("TOPLEFT", Popup, -1, 1)
	PopupBorder:SetPoint("BOTTOMRIGHT", Popup, 1, -1)
	PopupBorder:SetFrameStrata("BACKGROUND")
	PopupBorder:SetFrameLevel(1)
	PopupBorder:SetBackdrop(BackdropBorder)
	PopupBorder:SetBackdropBorderColor(0, 0, 0)
end

local SetupDropDown = function()
	if GMUI_DropDown then
		return
	end
	
	local DropDown = CreateFrame("Frame", "GMUI_DropDown", GMUI_GUI)
	DropDown:SetSize(120, 20)
	DropDown:SetPoint("TOPRIGHT", GMUI_GUI.Top, "BOTTOMRIGHT", -4, -49)
	SetTemplateDark(DropDown)
	
	DropDown.Label = DropDown:CreateFontString(nil, "OVERLAY")
	DropDown.Label:SetPoint("LEFT", DropDown, "LEFT", 4, -1.5)
	DropDown.Label:SetFont(Font, 16)
	DropDown.Label:SetTextColor(unpack(FontColor))
	DropDown.Label:SetShadowOffset(1.25, -1.25)
	DropDown.Label:SetShadowColor(0, 0, 0)
	
	DropDown.Button = CreateFrame("Frame", nil, DropDown)
	DropDown.Button:SetSize(20, 20)
	DropDown.Button:SetPoint("RIGHT", DropDown, "RIGHT", 0, 0)
	DropDown.Button.Pos = -1
	SetTemplate(DropDown.Button)
	
	DropDown.Button.Label = DropDown.Button:CreateFontString(nil, "OVERLAY")
	DropDown.Button.Label:SetPoint("CENTER", DropDown.Button, "CENTER", 1, -1.5)
	DropDown.Button.Label:SetFont(Font, 14)
	DropDown.Button.Label:SetTextColor(unpack(FontColor))
	DropDown.Button.Label:SetShadowOffset(1.25, -1.25)
	DropDown.Button.Label:SetShadowColor(0, 0, 0)
	DropDown.Button.Label:SetText("V")
	
	DropDown.Options = CreateFrame("Frame", nil, DropDown)
	DropDown.Options:SetSize(120, (NumGuildRanks * 20))
	DropDown.Options:SetPoint("TOP", DropDown, "BOTTOM", 0, -3)
	DropDown.Options:SetFrameStrata("HIGH")
	DropDown.Options:Hide()
	SetTemplate(DropDown.Options)
	
	DropDown.Button:SetScript("OnMouseUp", function(self)
		if (self.Pos == 1) then
			DropDown.Options:Hide()
			DropDown.Button.Label:SetText("V")
			
			self.Pos = -1
		else
			DropDown.Options:Show()
			DropDown.Button.Label:SetText("^")
			
			self.Pos = 1
		end
	end)
	
	for i = 1, NumGuildRanks do
		GMUI_DropDown.Options[i] = CreateFrame("Frame", nil, GMUI_DropDown.Options)
		GMUI_DropDown.Options[i]:SetSize(120, 20)
		GMUI_DropDown.Options[i]:SetPoint("TOP", GMUI_DropDown.Options, "TOP", 0, -(20 * (i - 1)))
		GMUI_DropDown.Options[i].RankIndex = (i - 1)
		
		GMUI_DropDown.Options[i]:SetScript("OnEnter", function(self) self.bg:SetTexture(67/255, 77/255, 91/255, 1) end)
		GMUI_DropDown.Options[i]:SetScript("OnLeave", function(self) self.bg:SetTexture(0, 0, 0, 0) end)
		
		GMUI_DropDown.Options[i]:SetScript("OnMouseUp", function(self)
			ActiveRankIndex = self.RankIndex
			DropDown.Label:SetText(self.Label:GetText())
			DropDown.Options:Hide()
			DropDown.Button.Label:SetText("V")
			DropDown.Button.Pos = -1
			GetNumEligableKicks()
		end)
		
		GMUI_DropDown.Options[i].bg = GMUI_DropDown.Options[i]:CreateTexture(nil, "OVERLAY")
		GMUI_DropDown.Options[i].bg:SetSize(118, 20)
		GMUI_DropDown.Options[i].bg:SetPoint("CENTER", GMUI_DropDown.Options[i], "CENTER", 0, 0)
		
		GMUI_DropDown.Options[i].Label = GMUI_DropDown.Options[i]:CreateFontString(nil, "OVERLAY")
		GMUI_DropDown.Options[i].Label:SetPoint("LEFT", GMUI_DropDown.Options[i], "LEFT", 4, -1.5)
		GMUI_DropDown.Options[i].Label:SetFont(Font, 16)
		GMUI_DropDown.Options[i].Label:SetTextColor(unpack(FontColor))
		GMUI_DropDown.Options[i].Label:SetShadowOffset(1.25, -1.25)
		GMUI_DropDown.Options[i].Label:SetShadowColor(0, 0, 0)
		GMUI_DropDown.Options[i].Label:SetText(GuildRanks[i-1])
	end
	
	-- Check for an existing rank, and set the default to 1 higher
	if (not DropDown.Label:GetText()) then
		for i = NumGuildRanks - 1, 0, -1 do
			if GuildRanks[i] then -- The reason for this check is because 0 players might be set to this rank index, so info wasn't retrieved
				ActiveRankIndex = i - 1
				break
			end
		end
		
		DropDown.Label:SetText(GuildRanks[ActiveRankIndex])
	end
end

ToggleGMUI_GUI = function()
	if (not GMUI_GUI) then
		UpdateGuildMembers()
		LoadGMUI_GUI()
		
		return
	end
	
	if GMUI_GUI:IsShown() then
		GMUI_GUI:Hide()
	else
		GMUI_GUI:Show()
		UpdateGuildMembers()
	end
end

local Frame = CreateFrame("Frame", nil, UIParent)
Frame:RegisterEvent("GUILD_ROSTER_UPDATE")
Frame:SetScript("OnEvent", function(self, event)
	-- Make sure we requested this update
	if Requested then
		BuildGuildRoster()
		UpdateGuildCount()
		SetupDropDown()
		GetNumEligableKicks()
		
		Requested = false
	end
end)

SLASH_GUILDMANAGERUI1 = "/gmui"
SlashCmdList["GUILDMANAGERUI"] = ToggleGMUI_GUI